'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var uuid = require('uuid');
var lodash = require('lodash');
var prosemirrorModel = require('prosemirror-model');
var inversify = require('inversify');
require('reflect-metadata');
var deepmerge = _interopDefault(require('deepmerge'));
var waxProsemirrorServices = require('wax-prosemirror-services');
var applyDevTools = _interopDefault(require('prosemirror-dev-tools'));
var prosemirrorState = require('prosemirror-state');
var prosemirrorView = require('prosemirror-view');
var prosemirrorHistory = require('prosemirror-history');
var prosemirrorDropcursor = require('prosemirror-dropcursor');
var prosemirrorGapcursor = require('prosemirror-gapcursor');

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(n);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

var WaxContext = React__default.createContext({
  view: {},
  activeView: {},
  activeViewId: null,
  app: null,
  updateView: null,
  updateActiveView: null,
  removeView: null
});
var WaxProvider = (function (props) {
  var _useState = React.useState({
    app: props.app,
    view: props.view || {},
    activeView: props.activeView || {},
    activeViewId: props.activeViewId || {},
    options: {
      fullScreen: false
    },
    transaction: {},
    setTransaction: function setTransaction(tr) {
      Object.assign(context.transaction, tr);
    },
    setOption: function setOption(option) {
      Object.assign(context.options, option);
    },
    removeView: function removeView(deletedView) {
      delete context.view[deletedView];
    },
    updateView: function updateView(newView, activeViewId) {
      var view = Object.assign(context.view, newView);
      var activeView = view[activeViewId || context.activeViewId];
      setContext(_objectSpread2(_objectSpread2({}, context), {}, {
        view: view,
        activeView: activeView,
        activeViewId: activeViewId || context.activeViewId
      }));
    }
  }),
      _useState2 = _slicedToArray(_useState, 2),
      context = _useState2[0],
      setContext = _useState2[1];

  return /*#__PURE__*/React__default.createElement(WaxContext.Provider, {
    value: _objectSpread2({}, context)
  }, props.children);
});
var useInjection = function useInjection(identifier) {
  var _useContext = React.useContext(WaxContext),
      container = _useContext.app.container;

  if (!container) {
    throw new Error();
  }

  return container.isBound(identifier) ? {
    instance: container.get(identifier)
  } : null;
};

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

var PortalContext = React__default.createContext({
  createPortal: function createPortal() {},
  portals: {}
});
var PortalProvider = (function (props) {
  var _useState = React.useState({
    createPortal: function createPortal(element, component, node, view, getPos, decorations, context) {
      setTimeout(function () {
        portal.portals.push({
          id: uuid.v4(),
          element: element,
          component: component,
          node: node,
          view: view,
          getPos: getPos,
          decorations: decorations,
          context: context
        });
        setPortal({
          createPortal: portal.createPortal,
          portals: _toConsumableArray(portal.portals)
        });
      });
    },
    portals: []
  }),
      _useState2 = _slicedToArray(_useState, 2),
      portal = _useState2[0],
      setPortal = _useState2[1];

  return /*#__PURE__*/React__default.createElement(PortalContext.Provider, {
    value: _objectSpread2({}, portal)
  }, props.children);
});

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

var ComponentPlugin = function ComponentPlugin(renderArea) {
  return function (layoutProps) {
    var _useInjection = useInjection('Layout'),
        instance = _useInjection.instance;

    var components = instance.render(renderArea);
    return components ? components.map(function (_ref, key) {
      var Component = _ref.component,
          componentProps = _ref.componentProps;
      return /*#__PURE__*/React__default.createElement(Component, Object.assign({
        key: "".concat(renderArea, "-").concat(key)
      }, layoutProps, componentProps));
    }) : null;
  };
};

var _dec, _class, _temp;
var Config = (_dec = inversify.injectable(), _dec(_class = (_temp = /*#__PURE__*/function () {
  function Config(config) {
    _classCallCheck(this, Config);

    this._config = {};
    this._config = config;
  }

  Config = inversify.inject('config')(Config, undefined, 0) || Config;

  _createClass(Config, [{
    key: "set",
    value: function set(key, value) {
      lodash.set(this._config, key, value);

      return this._config;
    }
  }, {
    key: "get",
    value: function get(key) {
      return lodash.get(this._config, key);
    }
  }, {
    key: "pushToArray",
    value: function pushToArray(key, value) {
      var oldValue = this.get(key);

      if (oldValue) {
        if (lodash.isArrayLikeObject(value)) {
          value.forEach(function (v) {
            oldValue.push(v);
          });
        } else {
          oldValue.push(value);
        }
      } else {
        oldValue = value;
      }

      this.set(key, oldValue);
      return this;
    }
  }]);

  return Config;
}(), _temp)) || _class);

var defaultConfig = (function () {
  return {
    services: [new waxProsemirrorServices.SchemaService(), new waxProsemirrorServices.RulesService(), new waxProsemirrorServices.ShortCutsService(), new waxProsemirrorServices.LayoutService(), new waxProsemirrorServices.PortalService(), new waxProsemirrorServices.MenuService(), new waxProsemirrorServices.OverlayService()]
  };
});

var _dec$1, _class$1, _temp$1;
var PmPlugins = (_dec$1 = inversify.injectable(), _dec$1(_class$1 = (_temp$1 = /*#__PURE__*/function () {
  function PmPlugins() {
    _classCallCheck(this, PmPlugins);

    this._plugins = new Map();
  }

  _createClass(PmPlugins, [{
    key: "add",
    value: function add(key, plugin) {
      this._plugins.set(key, plugin);
    }
  }, {
    key: "getAll",
    value: function getAll() {
      return _toConsumableArray(this._plugins.values());
    }
  }, {
    key: "get",
    value: function get(key) {
      return this._plugins.get(key);
    }
  }]);

  return PmPlugins;
}(), _temp$1)) || _class$1);

var Application = /*#__PURE__*/function () {
  function Application(container) {
    _classCallCheck(this, Application);

    this.container = container;
    this.PmPlugins = container.get('PmPlugins');
  }

  _createClass(Application, [{
    key: "registerServices",
    value: function registerServices() {
      var count = 0;

      while (count < this.config.get('config.services').length) {
        var allServices = this.config.get('config.services');
        var service = this.config.get('config.services')[count];
        /*
          set App to every service
          so services can have access to containers and config
          */

        service.setApp(this);

        if (service.dependencies) {
          var servicePos = count;
          allServices.splice.apply(allServices, [servicePos + 1, 0].concat(_toConsumableArray(service.dependencies)));
        }

        if (service.register) {
          service.register();
        }

        count += 1;
      }
    }
  }, {
    key: "setConfig",
    value: function setConfig() {
      this.config = this.container.get('Config');
    }
  }, {
    key: "setContext",
    value: function setContext(context) {
      this.context = context;
    }
  }, {
    key: "bootServices",
    value: function bootServices() {
      var services = this.config.get('config.services');
      services.forEach(function (plugin) {
        if (plugin.boot) {
          plugin.boot();
        }
      });
    }
  }, {
    key: "getPlugins",
    value: function getPlugins() {
      return this.PmPlugins.getAll();
    }
  }, {
    key: "getSchema",
    value: function getSchema() {
      this.schema = this.container.get('Schema');
      return this.schema.getSchema();
    }
  }, {
    key: "resetApp",
    value: function resetApp() {
      this.container = {};
      this.config = {};
      this.PmPlugins = {};
      this.schema = {};
    }
  }], [{
    key: "create",
    value: function create(config) {
      /* Merge Core Config with User Config */
      var appConfig = deepmerge({
        config: defaultConfig()
      }, config, {
        customMerge: function customMerge(key) {
          if (key === 'services') {
            return function (coreService, configService) {
              return coreService.concat(configService);
            };
          }
        }
      });
      /*
      Create Container
      */

      var container = new inversify.Container();
      /*
       Set base bindings for the App to work
      */

      container.bind('PmPlugins').to(PmPlugins).inSingletonScope();
      container.bind('Wax').toFactory(function () {
        return new Application(container);
      });
      container.bind('config').toConstantValue(appConfig);
      container.bind('Config').to(Config).inSingletonScope();
      /*
      Start the App
      */

      var app = container.get('Wax');
      app.setConfig();

      if (appConfig.config.PmPlugins) {
        appConfig.config.PmPlugins.forEach(function (configPlugin) {
          app.PmPlugins.add(configPlugin.key, configPlugin);
        });
      }

      app.registerServices();
      return app;
    }
  }]);

  return Application;
}();

var placeHolderText = new prosemirrorState.PluginKey('placeHolderText');
var Placeholder = (function (props) {
  return new prosemirrorState.Plugin({
    key: placeHolderText,
    props: {
      decorations: function decorations(state) {
        var decorations = [];

        var decorate = function decorate(node, pos) {
          if (node.type.isBlock && node.childCount === 0 && state.doc.content.childCount === 1) {
            decorations.push(prosemirrorView.Decoration.node(pos, pos + node.nodeSize, {
              "class": 'empty-node',
              'data-content': props.content
            }));
          }
        };

        state.doc.descendants(decorate);
        return prosemirrorView.DecorationSet.create(state.doc, decorations);
      }
    }
  });
});

var defaultPlugins = [prosemirrorDropcursor.dropCursor(), prosemirrorGapcursor.gapCursor(), prosemirrorHistory.history()];

var parser = function parser(schema) {
  var WaxParser = prosemirrorModel.DOMParser.fromSchema(schema);
  return function (content) {
    var container = document.createElement('article');
    container.innerHTML = content;
    return WaxParser.parse(container);
  };
};

var WaxOptions = (function (_ref) {
  var placeholder = _ref.placeholder,
      targetFormat = _ref.targetFormat,
      value = _ref.value,
      schema = _ref.schema,
      plugins = _ref.plugins;
  var finalPlugins = []; // eslint-disable-next-line no-shadow

  var createPlaceholder = function createPlaceholder(placeholder) {
    return Placeholder({
      content: placeholder
    });
  };

  finalPlugins = defaultPlugins.concat([createPlaceholder(placeholder)].concat(_toConsumableArray(plugins)));
  var WaxOptions = {
    schema: schema,
    plugins: finalPlugins
  };

  if (targetFormat === 'JSON') {
    var editorContent = value || {
      type: 'doc',
      content: [{
        type: 'paragraph',
        content: [{
          type: 'text',
          text: ' '
        }]
      }]
    };
    WaxOptions.doc = schema.nodeFromJSON(editorContent);
  } else {
    var _editorContent = value || '';

    var parse = parser(schema);
    WaxOptions.doc = parse(_editorContent);
  }

  return WaxOptions;
});

/* eslint-disable consistent-return */

var getDocContent = function getDocContent(schema, serializer, targetFormat, context) {
  /* HACK  alter toDOM of footnote, because of how PM treats inline nodes
      with content */
  var content = '';
  var notes = [];
  lodash.each(schema.nodes, function (node) {
    if (node.groups.includes('notes')) notes.push(node);
  });

  if (notes.length > 0) {
    notes.forEach(function (note) {
      schema.nodes[note.name].spec.toDOM = function (singleNode) {
        if (singleNode) return [note.name, singleNode.attrs, 0];
      };
    });
  }

  if (targetFormat === 'JSON') {
    content = context.app.context.view.main.state.doc.content;
  } else {
    var serialize = serializer(schema);
    content = serialize(context.app.context.view.main.state.doc.content);
  }

  if (notes.length > 0) {
    notes.forEach(function (note) {
      schema.nodes[note.name].spec.toDOM = function (sinlgeNode) {
        if (sinlgeNode) return [note.name, sinlgeNode.attrs];
      };
    });
  }

  return content;
};

var WaxPortals = ComponentPlugin('waxPortals');
var WaxOverlays = ComponentPlugin('waxOverlays');
var previousDoc;
var WaxView = React.forwardRef(function (props, ref) {
  var view;
  var browserSpellCheck = props.browserSpellCheck,
      customValues = props.customValues,
      readonly = props.readonly,
      debug = props.debug,
      autoFocus = props.autoFocus,
      user = props.user,
      targetFormat = props.targetFormat,
      serializer = props.serializer;
  var WaxEditorRef = React.useRef();

  var _useState = React.useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      mounted = _useState2[0],
      setMounted = _useState2[1];

  var context = React.useContext(WaxContext);

  var _useContext = React.useContext(PortalContext),
      createPortal = _useContext.createPortal;

  context.app.setContext(_objectSpread2(_objectSpread2({}, context), {}, {
    createPortal: createPortal
  }));
  var schema = context.app.getSchema();

  if (!mounted) {
    context.app.bootServices();
  }

  var setEditorRef = React.useCallback( // eslint-disable-next-line consistent-return
  function (node) {
    if (WaxEditorRef.current) ;

    if (node) {
      var options = WaxOptions(_objectSpread2(_objectSpread2({}, props), {}, {
        schema: schema,
        plugins: context.app.getPlugins()
      }));
      view = new prosemirrorView.EditorView({
        mount: node
      }, {
        editable: function editable() {
          return !readonly;
        },
        customValues: customValues,
        state: prosemirrorState.EditorState.create(options),
        dispatchTransaction: dispatchTransaction,
        disallowedTools: [],
        user: user,
        scrollMargin: 200,
        scrollThreshold: 200,
        attributes: {
          spellcheck: browserSpellCheck ? 'true' : 'false'
        }
      });
      setMounted(true);
      context.updateView({
        main: view
      }, 'main');
      if (debug) applyDevTools(view);
      if (autoFocus) setTimeout(function () {
        if (view) view.focus();
      }, 1000);
      return function () {
        return view.destroy();
      };
    }

    WaxEditorRef.current = node;
  }, [readonly, customValues]);
  React.useEffect(function () {
    return function () {
      return view = null;
    };
  }, []);
  React.useImperativeHandle(ref, function () {
    return {
      getContent: function getContent() {
        return getDocContent(schema, serializer, targetFormat, context);
      }
    };
  });

  var dispatchTransaction = function dispatchTransaction(transaction) {
    var TrackChange = props.TrackChange;
    var tr = TrackChange && TrackChange.enabled ? waxProsemirrorServices.trackedTransaction(transaction, view.state, user) : transaction;
    previousDoc = view.state.doc;
    var state = view.state.apply(tr);
    view.updateState(state);
    /* when a transaction comes from a view other than
      main don't keep updating the view ,as this is
      the central point of each transaction
      */

    context.setTransaction(transaction);

    if (!transaction.getMeta('outsideView')) {
      context.updateView({
        main: view
      }, 'main');
    }

    var docContent = targetFormat === 'JSON' ? state.doc.toJSON() : state.doc.content;
    if (!previousDoc.eq(view.state.doc) || tr.getMeta('forceUpdate')) props.onChange(docContent);
  };

  var editor = /*#__PURE__*/React__default.createElement(React__default.Fragment, null, /*#__PURE__*/React__default.createElement("div", {
    ref: setEditorRef
  }), context.activeViewId === 'main' && /*#__PURE__*/React__default.createElement(WaxOverlays, {
    group: "main"
  }), /*#__PURE__*/React__default.createElement(WaxPortals, null));
  return React.useMemo(function () {
    return props.children({
      editor: editor
    });
  }, [readonly, customValues, context.activeViewId]);
});

var serializer = function serializer(schema) {
  var WaxSerializer = prosemirrorModel.DOMSerializer.fromSchema(schema);
  return function (content) {
    var container = document.createElement('article');
    container.appendChild(WaxSerializer.serializeFragment(content));
    return container.innerHTML;
  };
};

var createApplication = function createApplication(props) {
  var application = Application.create(props);
  return application;
};

var Wax = React.forwardRef(function (props, ref) {
  var _useState = React.useState(),
      _useState2 = _slicedToArray(_useState, 2),
      application = _useState2[0],
      setApplication = _useState2[1];

  React.useEffect(function () {
    var newApplication = createApplication(props);
    setApplication(newApplication);
    return function () {
      return newApplication.resetApp();
    };
  }, []);
  var autoFocus = props.autoFocus,
      browserSpellCheck = props.browserSpellCheck,
      className = props.className,
      customValues = props.customValues,
      debug = props.debug,
      fileUpload = props.fileUpload,
      layout = props.layout,
      placeholder = props.placeholder,
      readonly = props.readonly,
      value = props.value,
      user = props.user,
      onChange = props.onChange,
      targetFormat = props.targetFormat;
  if (!application) return null;

  var WaxOnchange = onChange || function (v) {
    return true;
  };

  var finalOnChange = function finalOnChange(content) {
    /* HACK  alter toDOM of footnote, because of how PM treats inline nodes
      with content */
    var schema = application.schema.schema;
    var notes = [];
    lodash.each(schema.nodes, function (node) {
      if (node.groups.includes('notes')) notes.push(node);
    });

    if (notes.length > 0) {
      notes.forEach(function (note) {
        schema.nodes[note.name].spec.toDOM = function (node) {
          // eslint-disable-next-line prefer-rest-params
          if (node) return [note.name, node.attrs, 0];
        };
      });
    }

    if (targetFormat === 'JSON') {
      WaxOnchange(content);
    } else {
      var serialize = serializer(schema);
      WaxOnchange(serialize(content));
    }

    if (notes.length > 0) {
      notes.forEach(function (note) {
        schema.nodes[note.name].spec.toDOM = function (node) {
          // eslint-disable-next-line prefer-rest-params
          if (node) return [note.name, node.attrs];
        };
      });
    }
  };

  var TrackChange = application.config.get('config.EnableTrackChangeService');
  var Layout = application.container.get('Layout');
  if (layout) Layout.setLayout(layout);
  var WaxRender = Layout.layoutComponent;
  return /*#__PURE__*/React__default.createElement(WaxProvider, {
    app: application
  }, /*#__PURE__*/React__default.createElement(PortalProvider, null, /*#__PURE__*/React__default.createElement(WaxView, {
    autoFocus: autoFocus,
    browserSpellCheck: browserSpellCheck,
    customValues: customValues,
    debug: debug,
    fileUpload: fileUpload,
    onChange: finalOnChange || function (v) {
      return true;
    },
    placeholder: placeholder,
    readonly: readonly,
    ref: ref,
    serializer: serializer,
    targetFormat: targetFormat,
    TrackChange: TrackChange,
    user: user,
    value: value
  }, function (_ref) {
    var editor = _ref.editor;
    return /*#__PURE__*/React__default.createElement(WaxRender, {
      className: className,
      editor: editor
    });
  })));
});
Wax.defaultProps = {
  config: {
    services: []
  }
};

exports.ComponentPlugin = ComponentPlugin;
exports.PortalContext = PortalContext;
exports.Wax = Wax;
exports.WaxContext = WaxContext;
exports.useInjection = useInjection;
//# sourceMappingURL=index.js.map
